@@ -57,7 +57,7 @@ maybe_add_gsub_feature (PangoOTRuleset *ruleset,
 			gulong          property_bit)
 {
   guint feature_index;
-  
+
   /* 0xffff == default language system */
   if (pango_ot_info_find_feature (info, PANGO_OT_TABLE_GSUB,
 				  tag, script_index, 0xffff, &feature_index))
@@ -73,7 +73,7 @@ maybe_add_gpos_feature (PangoOTRuleset *ruleset,
 			gulong          property_bit)
 {
   guint feature_index;
-  
+
   /* 0xffff == default language system */
   if (pango_ot_info_find_feature (info, PANGO_OT_TABLE_GPOS,
 				  tag, script_index, 0xffff, &feature_index))
@@ -91,7 +91,7 @@ get_ruleset (FT_Face face)
 
   if (!ruleset_quark)
     ruleset_quark = g_quark_from_string ("pango-arabic-ruleset");
-  
+
   if (!info)
     return NULL;
 
@@ -148,16 +148,16 @@ static void
 swap_range (PangoGlyphString *glyphs, int start, int end)
 {
   int i, j;
-  
+
   for (i = start, j = end - 1; i < j; i++, j--)
     {
       PangoGlyphInfo glyph_info;
       gint log_cluster;
-      
+
       glyph_info = glyphs->glyphs[i];
       glyphs->glyphs[i] = glyphs->glyphs[j];
       glyphs->glyphs[j] = glyph_info;
-      
+
       log_cluster = glyphs->log_clusters[i];
       glyphs->log_clusters[i] = glyphs->log_clusters[j];
       glyphs->log_clusters[j] = log_cluster;
@@ -171,7 +171,7 @@ set_glyph (PangoFont *font, PangoGlyphString *glyphs, int i, int offset, PangoGl
   glyphs->log_clusters[i] = offset;
 }
 
-static void 
+static void
 fallback_shape (PangoEngineShape *engine,
 		PangoFont        *font,
 		const char       *text,
@@ -183,28 +183,25 @@ fallback_shape (PangoEngineShape *engine,
   glong n_chars = g_utf8_strlen (text, length);
   const char *p;
   int i;
-  
+
   pango_glyph_string_set_size (glyphs, n_chars);
   p = text;
-  
+
   for (i=0; i < n_chars; i++)
     {
       gunichar wc;
       gunichar mirrored_ch;
       PangoGlyph index;
       char buf[6];
-      const char *input;
 
       wc = g_utf8_get_char (p);
 
-      input = p;
       if (analysis->level % 2)
 	if (pango_get_mirror_char (wc, &mirrored_ch))
 	  {
 	    wc = mirrored_ch;
-	    
+
 	    g_unichar_to_utf8 (wc, buf);
-	    input = buf;
 	  }
 
       if (pango_is_zero_width (wc))
@@ -220,7 +217,7 @@ fallback_shape (PangoEngineShape *engine,
 
 	  set_glyph (font, glyphs, i, p - text, index);
 	}
-      
+
       p = g_utf8_next_char (p);
     }
 
@@ -230,17 +227,17 @@ fallback_shape (PangoEngineShape *engine,
       if (glyphs->glyphs[i].glyph)
 	{
 	  PangoRectangle logical_rect;
-	  
+
 	  pango_font_get_glyph_extents (font, glyphs->glyphs[i].glyph, NULL, &logical_rect);
 	  glyphs->glyphs[i].geometry.width = logical_rect.width;
 	}
       else
 	glyphs->glyphs[i].geometry.width = 0;
-      
+
       glyphs->glyphs[i].geometry.x_offset = 0;
       glyphs->glyphs[i].geometry.y_offset = 0;
     }
-  
+
   if (analysis->level % 2 != 0)
     {
       /* Swap all glyphs */
@@ -248,7 +245,7 @@ fallback_shape (PangoEngineShape *engine,
     }
 }
 
-static void 
+static void
 arabic_engine_shape (PangoEngineShape *engine,
 		     PangoFont        *font,
 		     const char       *text,
@@ -287,14 +284,14 @@ arabic_engine_shape (PangoEngineShape *engine,
   buffer = pango_ot_buffer_new (fc_font);
   pango_ot_buffer_set_rtl (buffer, analysis->level % 2 != 0);
   pango_ot_buffer_set_zero_width_marks (buffer, TRUE);
-    
+
   wcs = g_utf8_to_ucs4_fast (text, length, &n_chars);
   properties = g_new0 (gulong, n_chars);
-      
+
   Arabic_Assign_Properties (wcs, properties, n_chars);
 
   g_free (wcs);
-  
+
   p = text;
   for (i=0; i < n_chars; i++)
     {
@@ -302,18 +299,15 @@ arabic_engine_shape (PangoEngineShape *engine,
       gunichar mirrored_ch;
       PangoGlyph index;
       char buf[6];
-      const char *input;
 
       wc = g_utf8_get_char (p);
 
-      input = p;
       if (analysis->level % 2)
 	if (pango_get_mirror_char (wc, &mirrored_ch))
 	  {
 	    wc = mirrored_ch;
-	    
+
 	    g_unichar_to_utf8 (wc, buf);
-	    input = buf;
 	  }
 
       if (pango_is_zero_width (wc))	/* Zero-width characters */
@@ -331,7 +325,7 @@ arabic_engine_shape (PangoEngineShape *engine,
 	  if (wc == 0x6cc && ruleset && pango_fc_font_get_glyph (fc_font, 0x64a) &&
 	      ((properties[i] & (initial | medial)) != (initial | medial)))
 	    wc = 0x64a;
-	  
+
 	  index = pango_fc_font_get_glyph (fc_font, wc);
 
 	  if (!index)
@@ -343,19 +337,19 @@ arabic_engine_shape (PangoEngineShape *engine,
 	    {
 	      if (g_unichar_type (wc) != G_UNICODE_NON_SPACING_MARK)
 		cluster = p - text;
-		    
+
 	      pango_ot_buffer_add_glyph (buffer, index,
 					 properties[i], cluster);
 	    }
 	}
-      
+
       p = g_utf8_next_char (p);
     }
 
   pango_ot_ruleset_substitute (ruleset, buffer);
   pango_ot_ruleset_position (ruleset, buffer);
   pango_ot_buffer_output (buffer, glyphs);
-  
+
   g_free (properties);
   pango_ot_buffer_destroy (buffer);
 
@@ -370,20 +364,20 @@ arabic_engine_fc_class_init (PangoEngineShapeClass *class)
 }
 
 PANGO_ENGINE_SHAPE_DEFINE_TYPE (ArabicEngineFc, arabic_engine_fc,
-				arabic_engine_fc_class_init, NULL);
+				arabic_engine_fc_class_init, NULL)
 
-void 
+void
 PANGO_MODULE_ENTRY(init) (GTypeModule *module)
 {
   arabic_engine_fc_register_type (module);
 }
 
-void 
+void
 PANGO_MODULE_ENTRY(exit) (void)
 {
 }
 
-void 
+void
 PANGO_MODULE_ENTRY(list) (PangoEngineInfo **engines,
 			  int              *n_engines)
 {
