@@ -31,6 +31,8 @@ pango_attr_type_get_type (void)
       { PANGO_ATTR_UNDERLINE_COLOR, "PANGO_ATTR_UNDERLINE_COLOR", "underline-color" },
       { PANGO_ATTR_STRIKETHROUGH_COLOR, "PANGO_ATTR_STRIKETHROUGH_COLOR", "strikethrough-color" },
       { PANGO_ATTR_ABSOLUTE_SIZE, "PANGO_ATTR_ABSOLUTE_SIZE", "absolute-size" },
+      { PANGO_ATTR_GRAVITY, "PANGO_ATTR_GRAVITY", "gravity" },
+      { PANGO_ATTR_GRAVITY_HINT, "PANGO_ATTR_GRAVITY_HINT", "gravity-hint" },
       { 0, NULL, NULL }
     };
     etype = g_enum_register_static (g_intern_static_string ("PangoAttrType"), values);
@@ -155,6 +157,7 @@ pango_font_mask_get_type (void)
       { PANGO_FONT_MASK_WEIGHT, "PANGO_FONT_MASK_WEIGHT", "weight" },
       { PANGO_FONT_MASK_STRETCH, "PANGO_FONT_MASK_STRETCH", "stretch" },
       { PANGO_FONT_MASK_SIZE, "PANGO_FONT_MASK_SIZE", "size" },
+      { PANGO_FONT_MASK_GRAVITY, "PANGO_FONT_MASK_GRAVITY", "gravity" },
       { 0, NULL, NULL }
     };
     etype = g_flags_register_static (g_intern_static_string ("PangoFontMask"), values);
@@ -162,6 +165,40 @@ pango_font_mask_get_type (void)
   return etype;
 }
 
+/* enumerations from "pango-gravity.h" */
+GType
+pango_gravity_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { PANGO_GRAVITY_SOUTH, "PANGO_GRAVITY_SOUTH", "south" },
+      { PANGO_GRAVITY_EAST, "PANGO_GRAVITY_EAST", "east" },
+      { PANGO_GRAVITY_NORTH, "PANGO_GRAVITY_NORTH", "north" },
+      { PANGO_GRAVITY_WEST, "PANGO_GRAVITY_WEST", "west" },
+      { PANGO_GRAVITY_AUTO, "PANGO_GRAVITY_AUTO", "auto" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static (g_intern_static_string ("PangoGravity"), values);
+  }
+  return etype;
+}
+GType
+pango_gravity_hint_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { PANGO_GRAVITY_HINT_NATURAL, "PANGO_GRAVITY_HINT_NATURAL", "natural" },
+      { PANGO_GRAVITY_HINT_STRONG, "PANGO_GRAVITY_HINT_STRONG", "strong" },
+      { PANGO_GRAVITY_HINT_LINE, "PANGO_GRAVITY_HINT_LINE", "line" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static (g_intern_static_string ("PangoGravityHint"), values);
+  }
+  return etype;
+}
+
 /* enumerations from "pango-layout.h" */
 GType
 pango_alignment_get_type (void)
